package com.citygis.web.service.impl;

import cn.hutool.core.util.RandomUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.citygis.common.core.domain.AjaxResult;
import com.citygis.framework.web.service.TokenService;
import com.citygis.common.core.domain.entity.TabThirdPartyManage;
import com.citygis.web.mapper.TabThirdPartyManageMapper;
import com.citygis.web.service.ITabThirdPartyManageService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.concurrent.TimeUnit;

/**
 * <p>
 * 第三方系统管理 服务实现类
 * </p>
 *
 * @author citygis
 * @since 2025-02-25
 */
@Service
public class TabThirdPartyManageServiceImpl extends ServiceImpl<TabThirdPartyManageMapper, TabThirdPartyManage> implements ITabThirdPartyManageService {


    @Resource
    RedisTemplate redisTemplate;

    @Autowired
    private TokenService tokenService;


    @Override
    public AjaxResult getSysCode(TabThirdPartyManage tabThirdPartyManage) {
        String clientId = tabThirdPartyManage.getClientId();

        // 1. 验证 clientId 有效性
        LambdaQueryWrapper<TabThirdPartyManage> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TabThirdPartyManage::getClientId, clientId);
        TabThirdPartyManage one = this.getOne(queryWrapper);

        if (one == null) {
            return AjaxResult.error("clientId验证未通过");
        }

        // 2. 原子性生成并存储 code
        int code = RandomUtil.randomInt(1000, 10000);  // 确保4位数

        try {
            // 使用 setIfAbsent 避免并发问题
            Boolean isSet = redisTemplate.opsForValue().setIfAbsent(
                    clientId,
                    String.valueOf(code),
                    1,
                    TimeUnit.MINUTES
            );

            if (Boolean.TRUE.equals(isSet)) {
                return AjaxResult.success("操作成功,code有效期为1分钟").put("sysCode", code);
            } else {
                return AjaxResult.success("请稍等，正在获取系统CODE");
            }
        } catch (Exception e) {

            return AjaxResult.error("系统繁忙，请重试");
        }
    }

    @Override
    public AjaxResult getSysToken(TabThirdPartyManage tabThirdPartyManage) {

        String clientId = tabThirdPartyManage.getClientId();

        LambdaQueryWrapper<TabThirdPartyManage> queryWrapper = new LambdaQueryWrapper<>();
        queryWrapper.eq(TabThirdPartyManage::getClientId, clientId);
        queryWrapper.eq(TabThirdPartyManage::getClientSecret, tabThirdPartyManage.getClientSecret());

        TabThirdPartyManage one = this.getOne(queryWrapper);
        if (one == null) {
            return AjaxResult.error("clientId或clientSecret验证未通过");
        }
        if (!redisTemplate.hasKey(clientId)) {
            return AjaxResult.error("请先获取系统CODE");
        }
        String sysCode = (String) redisTemplate.opsForValue().get(clientId);
        if (!sysCode.equals(tabThirdPartyManage.getSysCode())) {
            return AjaxResult.error("请先获取系统CODE");
        }

        String token = tokenService.createToken(clientId+"123");


        return AjaxResult.success("操作成功").put("token", token).put("expiresIn", 1800);
    }
}
